import 'package:flutter/material.dart';

class CHFAppBar extends StatelessWidget implements PreferredSizeWidget {
  final String? title;
  final Widget? customTitle;
  final Widget? leading;
  final PreferredSizeWidget? bottom;
  final List<Widget>? actions;
  final Color? backgroundColor;
  final BorderSide? border;
  final bool? centerTitle;
  final Color? textColor;

  const CHFAppBar({
    Key? key,
    this.title,
    this.bottom,
    this.actions,
    this.backgroundColor,
    this.customTitle,
    this.leading,
    this.border,
    this.centerTitle: true,
    this.textColor,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Container(
      decoration: BoxDecoration(
        border: Border(bottom: border ?? BorderSide.none),
      ),
      child: AppBar(
        title: customTitle ??
            Text(
              title ?? " ",
              style: TextStyle(
                  color: textColor ?? const Color(0xff333333),
                  fontSize: 16,
                  fontWeight: FontWeight.bold),
            ),
        centerTitle: centerTitle,
        elevation: 0,
        leading: leading ??
            IconButton(
              icon: const BackButtonIcon(),
              splashRadius: 25,
              color: const Color(0xff333333),
              onPressed: () {
                Navigator.maybePop(context);
              },
            ),
        backgroundColor: backgroundColor,
        bottom: bottom,
        actions: actions,
      ),
    );
  }

  @override
  Size get preferredSize => Size.fromHeight(kToolbarHeight +
      (bottom?.preferredSize.height ?? 0.0) +
      (border == null ? 0.0 : border!.width));
}
