import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';

class TextExtendWidget extends StatefulWidget {
  final String text;
  final TextStyle style;
  final int maxLines;

  const TextExtendWidget(
      {Key? key,
        required this.text,
        required this.style,
        required this.maxLines})
      : super(key: key);

  @override
  _TextExtendWidgetState createState() => _TextExtendWidgetState();
}

class _TextExtendWidgetState extends State<TextExtendWidget> {
  bool expand = false;

  @override
  Widget build(BuildContext context) {
    return LayoutBuilder(builder: (context, size) {
      final span = TextSpan(text: widget.text, style: widget.style);
      final tp = TextPainter(
          text: span,
          maxLines: widget.maxLines,
          textDirection: TextDirection.ltr);
      tp.layout(maxWidth: size.maxWidth);
      if (tp.didExceedMaxLines) {
        return Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: <Widget>[
            expand
                ? Text(widget.text, style: widget.style)
                : Text(widget.text,
                maxLines: widget.maxLines,
                overflow: TextOverflow.ellipsis,
                style: widget.style),
            GestureDetector(
              behavior: HitTestBehavior.translucent,
              onTap: () {
                setState(() {
                  expand = !expand;
                });
              },
              child: Container(
                margin: EdgeInsets.only(top: 5),
                child: Text(expand ? '收起' : '点击查看',
                    style: TextStyle(
                        fontSize: 12,
                        height: 1.5,
                        color: Theme.of(context).primaryColor)),
              ),
            ),
          ],
        );
      } else {
        return Text(widget.text, style: widget.style);
      }
    });
  }
}
