import 'package:flutter/material.dart';

class CHFExpandableContainerWidget extends StatefulWidget {
  final Widget Function(BuildContext context)? headerBuilder;

  const CHFExpandableContainerWidget({
    Key? key,
    this.onExpansionChanged,
    this.headerBuilder,
    required this.child,
    this.initiallyExpanded = false,
    this.animationDuration,
    this.expandableController,
  }) : super(key: key);

  final ValueChanged<bool>? onExpansionChanged;

  final Widget child;

  final bool initiallyExpanded;

  final Duration? animationDuration;

  final CHFExpandableContainerController? expandableController;

  @override
  _CHFExpansionContainerElementState createState() =>
      _CHFExpansionContainerElementState();
}

class _CHFExpansionContainerElementState
    extends State<CHFExpandableContainerWidget>
    with SingleTickerProviderStateMixin {
  static final Animatable<double> _easeInTween =
      CurveTween(curve: Curves.easeIn);

  late CHFExpandableContainerController _expandableController;
  late AnimationController _animationController;
  late Animation<double> _heightFactor;

  bool _isExpanded = false;

  // Widget arrowIcon;

  @override
  void initState() {
    super.initState();
    _isExpanded =
        PageStorage.of(context)?.readState(context) ?? widget.initiallyExpanded;

    _expandableController =
        widget.expandableController ?? CHFExpandableContainerController();

    _expandableController.addListener(_expandableContainerControllerTick);
    _animationController = AnimationController(
        duration: widget.animationDuration ??
            Duration(milliseconds: 200) /*_kExpand*/,
        vsync: this);
    _heightFactor = _animationController.drive(_easeInTween);
    if (_isExpanded) {
      _animationController.value = 1.0;
    }
  }

  @override
  Widget build(BuildContext context) {
    final bool closed = !_isExpanded && _animationController.isDismissed;
    return AnimatedBuilder(
      animation: _animationController.view,
      builder: _buildHeader,
      child: closed ? null : widget.child,
    );
  }

  @override
  void dispose() {
    super.dispose();
    _expandableController.removeListener(_expandableContainerControllerTick);
    _animationController.dispose();
  }

  void _expandableContainerControllerTick() {
    if (_expandableController.expandableAction != CHFExpandableAction.none) {
      if (_expandableController.expandableAction ==
          CHFExpandableAction.toggle) {
        _handleTap();
      } else if (_isExpanded == false &&
          _expandableController.expandableAction ==
              CHFExpandableAction.expand) {
        _handleTap();
      } else if (_isExpanded == true &&
          _expandableController.expandableAction ==
              CHFExpandableAction.collapse) {
        _handleTap();
      }
    }
    _expandableController.expandableAction = CHFExpandableAction.none;
  }

  void _handleTap() {
    if (!mounted) return;
    setState(() {
      _isExpanded = !_isExpanded;
      if (_isExpanded) {
        _animationController.forward();
      } else {
        _animationController.reverse();
      }
      PageStorage.of(context)?.writeState(context, _isExpanded);
    });
    if (widget.onExpansionChanged != null)
      widget.onExpansionChanged!(_isExpanded);
  }

  Widget _buildHeader(BuildContext context, Widget? child) {
    late Widget content;
    if (widget.headerBuilder != null) {
      content = widget.headerBuilder!(context);
    } else {
      content = Container();
    }

    return Container(
      child: Column(
        mainAxisSize: MainAxisSize.min,
        children: <Widget>[
          Column(mainAxisSize: MainAxisSize.min, children: <Widget>[
            content,
          ]),

          // 可展开收起项
          Container(
            child: ClipRect(
              child: Align(
                heightFactor: _heightFactor.value,
                child: child,
              ),
            ),
          ),
        ],
      ),
    );
  }
}

class CHFExpandableContainerController extends ChangeNotifier {
  CHFExpandableAction expandableAction = CHFExpandableAction.none;

  void toggle() {
    expandableAction = CHFExpandableAction.toggle;
    notifyListeners();
  }

  void expand() {
    expandableAction = CHFExpandableAction.expand;
    notifyListeners();
  }

  void collapse() {
    expandableAction = CHFExpandableAction.collapse;
    notifyListeners();
  }
}

enum CHFExpandableAction {
  expand,
  collapse,
  toggle,
  none,
}
