import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

class CHField extends StatefulWidget {
  final TextEditingController controller;
  final TextInputType? keyboardType;
  final FocusNode? focusNode;
  final double height;
  final bool readonly;
  final bool clearable;
  final bool autofocus;
  final List<TextInputFormatter>? inputFormatters;
  final Widget? prefixWidget;
  final Widget? suffixWidget;
  final TextStyle? style;
  final String type;
  final int rows;
  final String? hintText;
  final TextStyle? hintStyle;
  final bool isDense;
  final bool showEyes;
  final Color? iconColor;
  final TextAlign? textAlign;
  final int? maxLength;
  final String? counterText;
  final TextStyle? counterStyle;
  final InputBorder? border;
  final InputBorder? enabledBorder;
  final InputBorder? focusBorder;
  final InputBorder? disabledBorder;
  final EdgeInsetsGeometry? contentPadding;
  final double? contentPaddingHorizontal;
  final BoxConstraints? prefixIconConstraints;
  final BoxConstraints? suffixIconConstraints;
  final bool filled;
  final Color? fillColor;
  final bool? showCursor;
  final Color? cursorColor;
  final double cursorWidth;
  final double? cursorHeight;
  final Radius? cursorRadius;
  final bool? enabled;
  final Widget? label;
  final TextStyle? labelStyle;
  final String? labelText;
  final TextStyle? floatingLabelStyle;
  final ValueChanged<String>? onSubmitted;
  final ValueChanged<String>? onChanged;
  final VoidCallback? onClearable;

  const CHField({
    Key? key,
    required this.controller,
    this.keyboardType,
    this.focusNode,
    this.height: 40,
    this.readonly: false,
    this.clearable: true,
    this.autofocus: false,
    this.inputFormatters,
    this.prefixWidget,
    this.suffixWidget,
    this.style,
    this.type: "text",
    this.rows: 2,
    this.hintText,
    this.hintStyle,
    this.isDense: true,
    this.showEyes: true,
    this.maxLength,
    this.counterText,
    this.counterStyle,
    this.iconColor,
    this.textAlign,
    this.border,
    this.enabledBorder,
    this.focusBorder,
    this.disabledBorder,
    this.contentPadding,
    this.contentPaddingHorizontal,
    this.prefixIconConstraints,
    this.suffixIconConstraints,
    this.fillColor,
    this.filled: false,
    this.showCursor,
    this.cursorColor,
    this.cursorWidth: 2.0,
    this.cursorHeight,
    this.cursorRadius,
    this.enabled,
    this.label,
    this.labelStyle,
    this.labelText,
    this.floatingLabelStyle,
    this.onSubmitted,
    this.onChanged,
    this.onClearable,
  }) : super(key: key);

  @override
  _CHFieldState createState() => _CHFieldState();
}

class _CHFieldState extends State<CHField> {
  bool _isShowPwd = false;
  bool _isShowDelete = false;

  @override
  void initState() {
    super.initState();

    _isShowDelete = widget.controller.text.isNotEmpty;

    widget.controller.addListener(() {
      if (mounted) {
        setState(() {
          _isShowDelete = widget.controller.text.isNotEmpty;
        });
      }
    });
  }

  Widget buildRight() {
    return Container(
      child: Row(
        children: <Widget>[
          widget.clearable && _isShowDelete
              ? GestureDetector(
                  child: Container(
                    margin: EdgeInsets.only(right: 10),
                    child: Icon(Icons.cancel,
                        size: 16.0,
                        color: widget.iconColor ?? const Color(0xffc8c9cc)),
                  ),
                  onTap: () {
                    widget.controller.clear();
                    widget.onClearable?.call();
                  },
                )
              : Container(),
          widget.type == "password" && widget.showEyes
              ? GestureDetector(
                  child: Container(
                    margin: EdgeInsets.only(right: 10),
                    child: Icon(
                        !_isShowPwd ? Icons.visibility_off : Icons.visibility,
                        size: 16.0,
                        color: widget.iconColor ?? const Color(0xff969799)),
                  ),
                  onTap: () {
                    setState(() {
                      _isShowPwd = !_isShowPwd;
                    });
                  },
                )
              : Container(),
          // if ((widget.clearable && _isShowDelete) ||
          //     (widget.type == "password" && widget.showEyes))
          //   SizedBox(width: 10)
        ],
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    TextStyle _style = widget.style ??
        TextStyle(
          fontSize: 12,
          color: Colors.black,
        );
    var _textPainter = TextPainter(
      text: TextSpan(
        text: '',
        style: _style,
      ),
      textDirection: TextDirection.ltr,
      textWidthBasis: TextWidthBasis.longestLine,
    )..layout();
    return TextField(
      maxLength: widget.maxLength,
      controller: widget.controller,
      focusNode: widget.focusNode,
      keyboardType: widget.keyboardType,
      autofocus: widget.autofocus,
      readOnly: widget.readonly,
      textAlign: widget.textAlign ?? TextAlign.start,
      maxLines: widget.type == "textarea" ? widget.rows : 1,
      style: _style,
      showCursor: widget.showCursor,
      cursorColor: widget.cursorColor ?? Theme.of(context).primaryColor,
      cursorWidth: widget.cursorWidth,
      cursorHeight: widget.cursorHeight,
      cursorRadius: widget.cursorRadius,
      inputFormatters: widget.inputFormatters,
      obscureText: widget.type == "password" ? !_isShowPwd : false,
      enabled: widget.enabled,
      decoration: InputDecoration(
        prefixIconConstraints: widget.prefixIconConstraints ??
            BoxConstraints(
              minHeight: widget.height,
              maxHeight: widget.height,
            ),
        suffixIconConstraints: widget.suffixIconConstraints ??
            BoxConstraints(
              minHeight: widget.height,
              maxHeight: widget.height,
            ),
        prefixIcon: widget.prefixWidget,
        suffixIcon: Row(
          mainAxisSize: MainAxisSize.min,
          children: [
            buildRight(),
            if (widget.suffixWidget != null) widget.suffixWidget!,
          ],
        ),
        contentPadding: widget.contentPadding ??
            EdgeInsets.symmetric(
              horizontal: widget.contentPaddingHorizontal ?? 10.0,
              vertical: (widget.height - _textPainter.height) / 2,
            ),
        isDense: widget.isDense,
        hintText: widget.hintText,
        counterText: widget.counterText,
        counterStyle: widget.counterStyle,
        hintStyle: widget.hintStyle ??
            TextStyle(
              color: const Color(0xFFD3D3D3),
              fontSize: 12,
            ),
        fillColor: widget.fillColor,
        filled: widget.filled,
        border: widget.border,
        enabledBorder: widget.enabledBorder,
        focusedBorder: widget.focusBorder,
        disabledBorder: widget.disabledBorder,
        label: widget.label,
        labelStyle: widget.labelStyle,
        labelText: widget.labelText,
        floatingLabelStyle: widget.floatingLabelStyle,
      ),
      onSubmitted: widget.onSubmitted,
      onChanged: widget.onChanged,
    );
  }
}
