import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';

class ExpandableTextConfig {
  static Color textColor = const Color(0xff333333);
  static String moreText = '更多';
  static String collapseText = '收起';
}

typedef TextExpandedCallback = Function(bool?);

class CHFExpandableText extends StatefulWidget {
  //显示的文本
  final String text;

  //显示的最多行数
  final int? maxLines;

  // 文本的样式
  final TextStyle? textStyle;

  // 展开或者收起的时候的回调
  final TextExpandedCallback? onExpanded;

  // 更多按钮渐变色的初始色 默认白色
  final Color? color;

  const CHFExpandableText(
      {Key? key,
        required this.text,
        this.maxLines = 1000000,
        this.textStyle,
        this.onExpanded,
        this.color})
      : super(key: key);

  _CHFExpandableTextState createState() => _CHFExpandableTextState();
}

class _CHFExpandableTextState extends State<CHFExpandableText> {
  bool? _expanded;

  @override
  void initState() {
    super.initState();
    _expanded = false;
  }

  @override
  Widget build(BuildContext context) {
    TextStyle style = _defaultTextStyle();
    return LayoutBuilder(
      builder: (context, size) {
        final span = TextSpan(text: widget.text, style: style);
        final tp = TextPainter(
            text: span,
            maxLines: widget.maxLines,
            textDirection: TextDirection.ltr,
            ellipsis: 'EllipseText');
        tp.layout(maxWidth: size.maxWidth);
        if (tp.didExceedMaxLines) {
          if (this._expanded!) {
            return _expandedText(context, widget.text);
          } else {
            return _foldedText(context, widget.text);
          }
        } else {
          return _regularText(widget.text, style);
        }
      },
    );
  }

  Widget _foldedText(context, String text) {
    return Stack(
      children: <Widget>[
        Text(
          widget.text,
          style: _defaultTextStyle(),
          maxLines: widget.maxLines,
          overflow: TextOverflow.clip,
        ),
        Positioned(
          bottom: 0,
          right: 0,
          child: _clickExpandTextWidget(context),
        )
      ],
    );
  }

  Widget _clickExpandTextWidget(context) {
    Color btnColor = widget.color ?? Colors.white;

    Text tx = Text(
     ExpandableTextConfig.moreText,
      style: TextStyle(
          color: Theme.of(context).primaryColor,
          fontSize: 14),
    );
    Container cnt = Container(
      padding: EdgeInsets.only(left: 22),
      alignment: Alignment.centerRight,
      child: tx,
      decoration: BoxDecoration(
          gradient: LinearGradient(
            colors: [btnColor.withAlpha(100), btnColor, btnColor],
            begin: Alignment.centerLeft,
            end: Alignment.centerRight,
          )),
    );
    return GestureDetector(
      child: cnt,
      onTap: () {
        setState(() {
          _expanded = true;
          if (null != widget.onExpanded) {
            widget.onExpanded!(_expanded);
          }
        });
      },
    );
  }

  Widget _expandedText(context, String text) {
    return RichText(
        textScaleFactor: MediaQuery.of(context).textScaleFactor,
        text: TextSpan(text: text, style: _defaultTextStyle(), children: [
          _foldButtonSpan(context),
        ]));
  }

  TextStyle _defaultTextStyle() {
    TextStyle style = widget.textStyle ??
        TextStyle(
          fontSize: 14,
          fontWeight: FontWeight.w400,
          color: ExpandableTextConfig.textColor,
        );
    return style;
  }

  InlineSpan _foldButtonSpan(context) {
    return TextSpan(
        text: '  ${ExpandableTextConfig.collapseText}',
        style: TextStyle(
          color: Theme.of(context).primaryColor,
          fontSize: 14,
        ),
        recognizer: TapGestureRecognizer()
          ..onTap = () {
            setState(() {
              _expanded = false;
              if (null != widget.onExpanded) {
                widget.onExpanded!(_expanded);
              }
            });
          });
  }

  Widget _regularText(text, style) {
    return Text(text, style: style);
  }
}
